// GSL Optimization Example in MS Visual Studio .Net 2003 (Visual C++ 7)
//
// This example is the ODE Example found in the GSL documentation:  
// http://www.gnu.org/software/gsl/manual/html_node/ODE-Example-programs.html
// 
// The program solves the second-order nonlinear Van der Pol oscillator equation:
//
//   x''(t) + \mu x'(t) (x(t)^2 - 1) + x(t) = 0                      (1)
//
// Equation (1) can be converted into a first order system of equations suitable for 
// use with the GSL ODE routines by introducing a separate variable for the velocity y = x'(t),
// and replacing this x'(t)=y in (1):
//
//     x' = y
//     y' = -x + \mu y (1-x^2)
//
// 2007/08/20 Miguel Torres Torriti
//
// --- Creating a project ---
//    Copy GSL include files to $MSVS/include/gsl
//    where $MSVS is the MS Visual C++ directory.
// 2. Copy GSL lib files to $MSVS/lib
//    where $MSVS is the MS Visual C++ directory.
// 3. Create a new project with the menu option:
//    File/New/Project/Visual C++ Projects/Win32/Win32 Console/Application Settings
//    -Select Console Application
//    -Select Empty Project
//4.a Add the file (gsl_ode_ex.h) to the project with the menu option
//    File/Add Existing Item or by right-clicking on "Header Files" within the Solution
//    Explorer pane (use CTRL+ALT+L to open the Solution Explorer) and then selecting 
//    Add/Add Existing Item...
//4.b Add the files (gsl_ode_ex.cpp, ode.cpp) to the project with the menu option
//    File/Add Existing Item or by right-clicking on "Source Files" within the Solution
//    Explorer pane (use CTRL+ALT+L to open the Solution Explorer) and then selecting 
//    Add/Add Existing Item...
// 5. Open this file (gsl_ode_ex.cpp) within the editor window and place the cursor within
//    the editor window.
// 6. Select the menu option Project/gsl_ode_ex Properties.  Set the following options:
//    a. For Debug version:
//       C/C++/Code Generation/Multi-threaded Debug DLL (/MDd)
//       Linker/General/Enable Incremental Linking = YES
//       Linker/Input/Additional Dependecies = gsl_d.lib
//       Linker/Input/Ignore Specific Library = libcd.lib
//    b. For Release version:
//       C/C++/Code Generation/Multi-threaded DLL (/MD)
//       Linker/General/Enable Incremental Linking = NO
//       Linker/Input/Additional Dependecies = gsl.lib
//       Linker/Input/Ignore Specific Library = libc.lib
//

#include <stdio.h>
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>
#include "gsl_ode_ex.h"

 
 int main (void)
 {
       const gsl_odeiv_step_type * T 
         = gsl_odeiv_step_rk8pd;
     
       gsl_odeiv_step * s 
         = gsl_odeiv_step_alloc (T, 2);
       gsl_odeiv_control * c 
         = gsl_odeiv_control_y_new (1e-6, 0.0);
       gsl_odeiv_evolve * e 
         = gsl_odeiv_evolve_alloc (2);
     
       double mu = 10;
       gsl_odeiv_system sys = {func, jac, 2, &mu};
     
       double t = 0.0, t1 = 100.0;
       double h = 1e-6;
       double y[2] = { 1.0, 0.0 };
     
       while (t < t1)
         {
           int status = gsl_odeiv_evolve_apply (e, c, s,
                                                &sys, 
                                                &t, t1,
                                                &h, y);
     
           if (status != GSL_SUCCESS)
               break;
     
           printf ("%.5e %.5e %.5e\n", t, y[0], y[1]);
         }
     
       gsl_odeiv_evolve_free (e);
       gsl_odeiv_control_free (c);
       gsl_odeiv_step_free (s);
       return 0;
     }
